@@ -542,16 +542,29 @@ subclass to the :HOSTATTRS subroutine of properties calling this."
   (alet (sh-escape (drop-trailing-slash (unix-namestring directory)))
     (mrun (format nil "rm -rf -- ~A/* ~A/.[!.]* ~A/..?*" it it it))))
 
-(defun remote-exists-p (&rest paths)
-  "Does each of PATHS exists?
-PATH may be any kind of file, including directories."
+(defun remote-test-multiple (operator paths connective)
   (apply #'remote-test (loop for path on paths
-                             nconc (list "-e" (car path))
-                             when (cdr path) collect "-a")))
+                             nconc (list operator (car path))
+                             when (cdr path) collect connective)))
+
+(defun remote-exists-p (&rest paths)
+  "Does each of PATHS exist?
+PATH may be any kind of file, including directories."
+  (remote-test-multiple "-e" paths "-a"))
+
+(defun remote-exists-every-p (&rest paths)
+  "Does each of PATHS exist?
+PATH may be any kind of file, including directories."
+  (remote-test-multiple "-e" paths "-a"))
+
+(defun remote-exists-some-p (&rest paths)
+  "Do any of PATHS exist?
+PATH may be any kind of file, including directories."
+  (remote-test-multiple "-e" paths "-o"))
 
 (defun remote-file-stats (path)
-  "Get the numeric mode, size in bytes and mtime of PATH, or NIL if it does not
-exist.
+  "Get the numeric mode, size in bytes, mtime, owner and group of PATH, or NIL if
+it does not exist.
 
 The mtime is only accurate to the nearest UTC day, rounding down, if the file
 was modified in the past six months or its mtime is in the future, and only
@@ -593,7 +606,9 @@ specification of POSIX ls(1))."
                             date month (nth-value 5 (get-decoded-time))
                             0))
                          (encode-universal-time
-                          0 0 0 date month (parse-integer (nth 7 ls)) 0))))))))
+                          0 0 0 date month (parse-integer (nth 7 ls)) 0)))
+                   (nth 2 ls)
+                   (nth 3 ls))))))
 
 (defun remote-last-reboot ()
   "Get the time of the last reboot, rounded down to the nearest minute."
