@@ -277,6 +277,8 @@ setgroups(2) is denied in the namespace."
                  (let ((owner (get-userns-owner (caar ns-fds))))
                    (if (zerop owner)
                        (nix:setgroups nil)
+                       ;; We can't use USER:USER-INFO here because we can't
+                       ;; run commands using RUNLINES.
                        (alet (osicat:user-info owner)
                          ;; As a precaution, we could also setuid & setgid to
                          ;; OWNER here.  However, it ought to be meaningless
@@ -299,12 +301,13 @@ setgroups(2) is denied in the namespace."
                ;; errors only of type EPERM, and see below about EPERM in the
                ;; second pass.
                (when setuserns
-                 (loop for cell on (cdr ns-fds)
-                       do (handler-case (setns (caar cell) (cdar cell))
-                            (nix:eperm ())
-                            (:no-error (&rest ignore)
-                              (declare (ignore ignore))
-                              (rplaca (car cell) nil)))))
+                 (mapl (lambda (ns-fds)
+                         (handler-case (setns (caar ns-fds) (cdar ns-fds))
+                           (nix:eperm ())
+                           (:no-error (&rest ignore)
+                             (declare (ignore ignore))
+                             (rplaca (car ns-fds) nil))))
+                       (cdr ns-fds)))
                (loop for (fd . type) in ns-fds
                      ;; Accept failures due to insufficient capabilities which
                      ;; occur after we've entered the new userns, as that
@@ -330,6 +333,8 @@ setgroups(2) is denied in the namespace."
                (nix:fchdir root-fd) (chroot ".")))
         (mapc #'nix:close opened-fds))
       (when uid
+        ;; We similarly can't use USER:USER-INFO here because we still can't
+        ;; run commands using RUNLINES.
         (alet (or (osicat:user-info uid)
                   (error "~&Could not look up user info for UID ~A." uid))
           (setf user (cdr (assoc :name it)))
