@@ -19,10 +19,14 @@
 (named-readtables:in-readtable :consfigurator)
 
 (defclass rehome-connection ()
-  ((datadir
-    :type :string :initarg :datadir :reader datadir
+  ((rehome-datadir
+    :type :string :initarg :rehome-datadir :reader rehome-datadir
     :documentation
-    "Where Consfigurator would cache items of prerequisite data in the new HOME."))
+    "Where Consfigurator would cache items of prerequisite data in the new HOME,
+as accessible from the previous connection hop.
+
+In the case of a connection which chroots, for example, this will be the path
+to a directory inside the chroot as seen from outside the chroot."))
   (:documentation
    "A connection which works by switching to a new HOME on the same host."))
 
@@ -31,10 +35,11 @@
     (upload-all-prerequisite-data connection))
 
 (defmethod connection-upload ((connection rehome-connection) (data file-data))
-  (with-slots (iden1 iden2 data-version) data
-    (let ((inside (data-pathname
-                   (datadir connection) iden1 iden2 data-version))
-          (outside (remote-data-pathname iden1 iden2 data-version)))
+  (with-slots (data-iden1 data-iden2 data-version) data
+    (let ((inside
+            (data-pathname
+                   (rehome-datadir connection) data-iden1 data-iden2 data-version))
+          (outside (remote-data-pathname data-iden1 data-iden2 data-version)))
       (mrun "mkdir" "-p" (pathname-directory-pathname inside))
       (if (remote-exists-p outside)
           (mrun "cp" outside inside)
@@ -48,9 +53,9 @@
 
 (defmethod connection-clear-data-cache
     ((connection rehome-connection) iden1 iden2)
-  (with-slots (datadir) connection
-    (delete-remote-trees (data-pathname (datadir connection) iden1 iden2))))
+  (delete-remote-trees
+   (data-pathname (rehome-datadir connection) iden1 iden2)))
 
 (defmethod get-remote-cached-prerequisite-data
     ((connection rehome-connection))
-  (get-local-cached-prerequisite-data (datadir connection)))
+  (get-local-cached-prerequisite-data (rehome-datadir connection)))
