@@ -25,7 +25,7 @@
     (error "~&SETUIDing requires a Lisp image running as root"))
   (informat 1 "~&SETUIDing to ~A" user)
   (let* ((ent
-           (or (osicat:user-info user)
+           (or (user:user-info user)
                (failed-change "~&Could not look up user info for ~A." user)))
 	 (xdg-cache-home
            (ensure-directory-pathname
