@@ -1,6 +1,6 @@
 ;;; Consfigurator -- Lisp declarative configuration management system
 
-;;; Copyright (C) 2021  Sean Whitton <spwhitton@spwhitton.name>
+;;; Copyright (C) 2021-2022  Sean Whitton <spwhitton@spwhitton.name>
 
 ;;; This file is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -21,20 +21,21 @@
 ;;;; Prerequisite data
 
 (defclass data ()
-  ((iden1
+  ((data-iden1
     :initarg :iden1
-    :reader iden1)
-   (iden2
+    :initform (simple-program-error "Must supply iden1 for data.")
+    :reader data-iden1)
+   (data-iden2
     :initarg :iden2
-    :reader iden2)
+    :initform (simple-program-error "Must supply iden2 for data.")
+    :reader data-iden2)
    (data-version
     :initarg :version
-    :reader data-version
-    :initform nil)
+    :initform (simple-program-error "Must supply version for data.")
+    :reader data-version)
    (data-mime
     :initarg :mime
     :accessor data-mime
-    :initform nil
     :documentation "The MIME type of the data, if known."))
   (:documentation
    "An item of prerequisite data as provided by a registered prerequisite data
@@ -134,10 +135,6 @@ as, before being passed to implementations of REGISTER-DATA-SOURCE."
       (push pair *data-sources*)
       (push args *data-source-registrations*))))
 
-(defun skip-data-source (c)
-  (declare (ignore c))
-  (invoke-restart 'skip-data-source))
-
 (defun reset-data-sources ()
   "Forget all data sources registered in this Lisp image and items of string
 data obtained from data sources by this Lisp image.
@@ -146,6 +143,15 @@ This function is typically called at the REPL."
         *data-sources* nil
         *data-source-registrations* nil))
 
+(defmacro with-reset-data-sources (&body body)
+  "Run BODY with initially empty data sources and string data.
+
+This macro is typically used for testing or debugging."
+  `(let ((*string-data* (make-hash-table))
+         *data-sources*
+         *data-source-registrations*)
+     ,@body))
+
 (defun get-data-string (iden1 iden2)
   "Return the content of an item of prerequisite data as a string.
 
@@ -173,6 +179,9 @@ This function is called by property :APPLY and :UNAPPLY subroutines."
                      (missing-iden1 condition) (missing-iden2 condition)))))
 
 (defun %get-data (iden1 iden2)
+  (alet (first-char iden1)
+    (unless (or (char= #\_ it) (char= #\- it) (valid-hostname-p iden1))
+      (simple-program-error "Invalid IDEN1: ~S" iden1)))
   (let* ((idenpair (cons iden1 iden2))
          (from-source (query-data-sources iden1 iden2))
          (from-source-version (and from-source (car from-source)))
@@ -183,7 +192,8 @@ This function is called by property :APPLY and :UNAPPLY subroutines."
                                  (and (string= (first c) iden1)
                                       (string= (second c) iden2)))
                                (sort-prerequisite-data-cache
-                                (get-local-cached-prerequisite-data)))))
+                                (get-local-cached-prerequisite-data
+                                 (get-local-data-cache-dir))))))
          (local-cached-version (caddr local-cached)))
     (cond
       ((and in-memory
@@ -258,14 +268,16 @@ This function is for implementation of REGISTER-DATA-SOURCE to check for
 clashes.  It should not be called by properties."
   (if (query-data-sources iden1 iden2) t nil))
 
-(defun maybe-writefile-data (path iden1 iden2 &key (mode nil mode-supplied-p))
-  "Wrapper around WRITEFILE which returns :NO-CHANGE and avoids touching PATH if
-PATH's content is already the prerequisite data identified by IDEN1 and IDEN2
-and PATH has mode MODE."
+(defun maybe-write-remote-file-data
+    (path iden1 iden2 &key (mode nil mode-supplied-p))
+  "Wrapper around WRITE-REMOTE-FILE which returns :NO-CHANGE and avoids touching
+PATH if PATH's content is already the prerequisite data identified by IDEN1
+and IDEN2 and PATH has mode MODE."
   (let ((data (funcall (%get-data iden1 iden2))))
     (etypecase data
-      (string-data (apply #'maybe-writefile-string path (data-string data)
-                          (and mode-supplied-p `(:mode ,mode))))
+      (string-data
+       (apply #'maybe-write-remote-file-string path (data-string data)
+              (and mode-supplied-p `(:mode ,mode))))
       (file-data
        (let ((stream (%get-data-stream data)))
          (if (and (remote-exists-p path)
@@ -275,40 +287,41 @@ and PATH has mode MODE."
                          (= (file-length stream) existing-size)
                          (= (data-cksum data) (cksum path)))))
              :no-change
-             (apply #'writefile path stream
+             (apply #'write-remote-file path stream
                     (and mode-supplied-p `(:mode ,mode)))))))))
 
 (defgeneric connection-upload (connection data)
   (:documentation
    "Subroutine to upload an item of prerequisite data to the remote cache.
 The default implementation will work for any connection which implements
-CONNECTION-WRITEFILE and CONNECTION-RUN, but connection types which work by
+CONNECTION-WRITE-FILE and CONNECTION-RUN, but connection types which work by
 calling CONTINUE-DEPLOY* or CONTINUE-DEPLOY*-PROGRAM will need their own
 implementation."))
 
 (defmethod connection-upload ((connection connection) (data data))
   (flet ((upload (from to)
            (with-open-file (stream from :element-type '(unsigned-byte 8))
-             (writefile to stream))))
-    (with-slots (iden1 iden2 data-version) data
-      (informat 1 "~&Uploading (~@{~S~^ ~}) ... " iden1 iden2 data-version)
+             (write-remote-file to stream))))
+    (with-slots (data-iden1 data-iden2 data-version) data
+      (informat 1 "~&Uploading (~@{~S~^ ~}) ... "
+                data-iden1 data-iden2 data-version)
       (let* ((*connection* connection)
-             (dest (remote-data-pathname iden1 iden2 data-version))
+             (dest (remote-data-pathname data-iden1 data-iden2 data-version))
              (destdir (pathname-directory-pathname dest))
              (destfile (pathname-file dest)))
         (mrun "mkdir" "-p" destdir)
         (with-remote-current-directory (destdir)
           (etypecase data
             (string-data
-             (writefile destfile (data-string data)))
+             (write-remote-file destfile (data-string data)))
             (file-data
              (let ((source (unix-namestring (data-file data))))
-               (if (string-prefix-p "text/" (data-mime data))
+               (if (and (slot-boundp data 'data-mime)
+                        (string-prefix-p "text/" (data-mime data)))
                    (let ((destfile (strcat destfile ".gz")))
                      (with-temporary-file (:pathname tmp)
                        (run-program
-                        (strcat "gzip -c " (escape-sh-token source))
-                        :output tmp)
+                        (strcat "gzip -c " (sh-escape source)) :output tmp)
                        (upload tmp destfile)
                        (mrun "gunzip" destfile)))
                    (upload source destfile)))))))))
@@ -370,11 +383,10 @@ CONTINUE-DEPLOY* or CONTINUE-DEPLOY*-PROGRAM."
           else do (error 'missing-data :iden1 iden1 :iden2 iden2))))
 
 (defun try-get-file-mime-type (file)
-  (handler-case (stripln (run-program
-                          (escape-sh-command (list "file" "-E"
-                                                   "--mime-type" "--brief"
-                                                   (unix-namestring file)))
-                          :output :string))
+  (handler-case (stripln
+                 (run-program
+                  (sh-escape `("file" "-E" "--mime-type" "--brief" ,file))
+                  :output :string))
     (subprocess-error () nil)))
 
 (defun sort-prerequisite-data-cache (cache)
@@ -382,7 +394,7 @@ CONTINUE-DEPLOY* or CONTINUE-DEPLOY*-PROGRAM."
 
 (defun data-pathname (root &rest segments)
   (destructuring-bind (last . rest)
-      (nreverse (mapcar #'string->filename segments))
+      (nreverse (mapcar #'string-to-filename segments))
     (merge-pathnames last (reduce #'merge-pathnames
                                   (mapcar (lambda (s) (strcat s "/")) rest)
                                   :from-end t :initial-value root))))
@@ -419,7 +431,7 @@ of CONNECTION, where each entry is of the form
 (defmethod get-remote-cached-prerequisite-data ((connection connection))
   (let ((*connection* connection))
     (mapcar (lambda (line)
-              (mapcar #'filename->string (split-string line :separator "/")))
+              (mapcar #'filename-to-string (split-string line :separator "/")))
             (multiple-value-bind (out exit)
                 (mrun :may-fail "find" (merge-pathnames
                                         "data/"
@@ -430,8 +442,7 @@ of CONNECTION, where each entry is of the form
 
 ;;;; Local caches
 
-(defun get-local-cached-prerequisite-data
-    (&optional (where (get-local-data-cache-dir)))
+(defun get-local-cached-prerequisite-data (where)
   "Scan a local cache of prerequisite data at WHERE, and return a list of
 items of prerequisite data where each entry is of the form
 
@@ -443,7 +454,7 @@ always supply a value for WHERE."
         nconc (loop for subdir in (subdirectories dir)
                     nconc (loop for file in (directory-files subdir)
                                 collect
-                                (mapcar #'filename->string
+                                (mapcar #'filename-to-string
                                         (list (lastcar
                                                (pathname-directory dir))
                                               (lastcar
@@ -462,7 +473,8 @@ properties, or data sources which return objects referencing existing files."
 			     (and (string= (car c) iden1)
 				  (string= (cadr c) iden2)))
 			   (sort-prerequisite-data-cache
-			    (get-local-cached-prerequisite-data))))))
+			    (get-local-cached-prerequisite-data
+                             (get-local-data-cache-dir)))))))
     (make-instance 'file-data :file (apply #'local-data-pathname triple)
 			      :iden1 (car triple)
 			      :iden2 (cadr triple)
@@ -484,26 +496,32 @@ properties, or data sources which return objects referencing existing files."
 
 ;;;; Passphrases
 
-(defclass passphrase ()
-  ((passphrase :initarg :passphrase :reader passphrase)))
+(defclass wrapped-passphrase ()
+  ((passphrase :initarg :passphrase :reader unwrap-passphrase)))
 
-(defun make-passphrase (passphrase)
+(defun wrap-passphrase (passphrase)
   "Make an object which is unprintable by default to contain a passphrase."
-  (make-instance 'passphrase :passphrase passphrase))
+  (make-instance 'wrapped-passphrase :passphrase passphrase))
 
 (defun get-data-protected-string (iden1 iden2)
   "Like GET-DATA-STRING, but wrap the content in an object which is unprintable
 by default.  Intended for code which fetches passwords and wants to lessen the
 chance of those passwords showing up in the clear in the Lisp debugger."
-  (make-passphrase (get-data-string iden1 iden2)))
+  (wrap-passphrase (get-data-string iden1 iden2)))
 
 (defvar *allow-printing-passphrases* nil)
 
-(defmethod print-object ((passphrase passphrase) stream)
+(defmethod print-object ((passphrase wrapped-passphrase) stream)
   (if *allow-printing-passphrases*
       (format stream "#.~S"
-              `(make-instance 'passphrase
-                              :passphrase ,(passphrase passphrase)))
+              `(make-instance 'wrapped-passphrase
+                              :passphrase ,(unwrap-passphrase passphrase)))
       (print-unreadable-object (passphrase stream)
         (format stream "PASSPHRASE")))
   passphrase)
+
+(defvar *data-source-gnupghome* nil
+  "Home directory for gnupg when used in a data source.
+
+Because gnupg uses Unix domain sockets internally, this path should be short
+enough to avoid the 108 char limit on socket paths.")
