@@ -432,6 +432,7 @@
                     #:has-ownership
                     #:does-not-exist
                     #:directory-does-not-exist
+                    #:empty-directory-does-not-exist
                     #:data-uploaded
                     #:host-data-uploaded
                     #:secret-uploaded
@@ -951,6 +952,7 @@
                              (#:systemd     #:consfigurator.property.systemd))
            (:export #:installed
                     #:user-container-started
+                    #:user-container-stopped
                     #:when-user-container-running
                     #:user-containers-autostart
                     #:usernet-veth-usable-by
