@@ -1,7 +1,7 @@
 ;;; Consfigurator -- Lisp declarative configuration management system
 
-;;; Copyright (C) 2021  David Bremner <david@tethera.net>
-;;; Copyright (C) 2021  Sean Whitton <spwhitton@spwhitton.name>
+;;; Copyright (C) 2021-2022  David Bremner <david@tethera.net>
+;;; Copyright (C) 2021-2022  Sean Whitton <spwhitton@spwhitton.name>
 
 ;;; This file is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -52,6 +52,16 @@ that group, and ~USERNAME and its contents are owned by UID:GID."
   (has-account username)
   (%has-uid-gid username uid gid))
 
+(defprop group-exists :posix (groupname)
+  "Ensure there is a group GROUPNAME.
+Note that this uses getent(1) and so is not strictly POSIX-compatible."
+  (:desc #?"Group ${groupname} exists")
+  (:check
+   (zerop (mrun :for-exit "getent" "group" groupname)))
+  (:apply
+   (assert-remote-euid-root)
+   (mrun "groupadd" groupname)))
+
 (defprop has-groups :posix
     (username &rest groups &aux (groups* (format nil "~{~A~^,~}" groups)))
   "Ensure that USERNAME is a member of secondary groups GROUPS."
@@ -141,3 +151,23 @@ properties not strictly POSIX-compatible."
 
 (defun user-exists (username)
   (zerop (mrun :for-exit "getent" "passwd" username)))
+
+(defun user-info (username-or-uid)
+  "Return passwd database entry for USERNAME-OR-UID as an alist.
+
+Falls back to getent(1), which is not specified in POSIX, so use of this
+function makes properties not strictly POSIX-compatible."
+  ;; getpwnam(3) and getpwuid(3) can fail to load the required NSS modules if
+  ;; we have chrooted or similar.  In that case, it appears as though the user
+  ;; does not exist.  So fall back to getent(1).
+  (or (and (lisp-connection-p) (osicat:user-info username-or-uid))
+      (aand (runlines "getent" "passwd" (aetypecase username-or-uid
+                                          (string it)
+                                          (number (write-to-string it))))
+            (destructuring-bind (name password uid gid &rest rest)
+                (split-string (car it) :separator '(#\:))
+              (declare (ignore password))
+              (list* (cons :name name)
+                     (cons :user-id (parse-integer uid))
+                     (cons :group-id (parse-integer gid))
+                     (pairlis '(:gecos :home :shell) rest))))))
