@@ -20,20 +20,23 @@
 
 (defun multiple-value-mapcan (function &rest lists)
   "Variant of MAPCAN which preserves multiple return values."
-  (let ((lists (copy-list lists))
-        (results (make-array '(2) :initial-element nil :adjustable t)))
-    (loop for new-results
-            = (multiple-value-list
-               (apply function
-                      (loop for list on lists
-                            if (car list)
-                              collect (pop (car list))
-                            else do (return-from multiple-value-mapcan
-                                      (values-list (coerce results 'list))))))
-          do (adjust-array results (max (length results) (length new-results))
-                           :initial-element nil)
-             (loop for result in new-results and i upfrom 0
-                   do (nconcf (aref results i) result)))))
+  (loop with lists = (copy-list lists)
+        with results = (make-array '(1) :initial-element nil :adjustable t)
+        for new = (multiple-value-list
+                   (apply function
+                          (maplist (lambda (lists)
+                                     (if (endp (car lists))
+                                         (return
+                                           (values-list
+                                            (map 'list #'nreverse results)))
+                                         (pop (car lists))))
+                                   lists)))
+        do (loop
+             initially
+                (adjust-array results (max (length results) (length new))
+                              :initial-element nil)
+             for result in new and i upfrom 0 do
+               (setf (aref results i) (nreconc result (aref results i))))))
 
 (defun lines (text &optional trimfun (trimchars '(#\Space #\Tab)))
   (with-input-from-string (stream text)
@@ -415,6 +418,35 @@ expansion as a starting point for your own DEFPACKAGE form for your consfig."
                    (re:scan "^[a-zA-Z0-9][a-zA-Z0-9-]*$" part)))
             parts))))
 
+(defmacro prog-changes (&body body)
+  (with-gensyms (result)
+    `(let ((,result :no-change))
+       (block prog-changes
+         (flet ((add-change (&optional result)
+                  (case result
+                    (:no-change result)
+                    (t (setq ,result result))))
+                (return-changes ()
+                  (return-from prog-changes ,result)))
+           (declare (ignorable #'return-changes))
+           ,@body
+           ,result)))))
+
+(defun sh-script-to-single-line (script)
+  "Attempt to convert a multiline POSIX sh script to a single line.
+
+The current implementation is naïve, and certainly unsuitable for converting
+arbitrary scripts.  Thus, this function is presently intended to be used only
+on simple scripts embedded in source code, written with newlines for the sake
+of maintainability.  Converting those scripts to single lines before they are
+executed improves Consfigurator's debug output, and also makes process names
+visible to remote commands like ps(1) more readable."
+  (re:regex-replace-all
+   #?/\s+/ (re:regex-replace-all "(then|else|elif|fi|case|in|;;|do|done);"
+                                 (format nil "~{~A~^; ~}" (lines script))
+                                 "\\1")
+   " "))
+
 
 ;;;; Progress & debug printing
 
